#!/usr/bin/python

import re
import seqtool
import sys

import alignment
from alignment import Alignment
from readtool import skipto

True, False = 1==1, 1!=1

def readblastz(filename, overlap, add=0.0, mul=1.0):
	f = open(filename)

	# check the file type
	m = skipto(f, r'#:lav')
	
	# end of the file
	assert m, 'File '+filename+' is not a blastz alignment file'
	
	re_tag = re.compile(r'\s*(\S)\s{([^}]+)}', re.MULTILINE)
	re_s_tag = re.compile(r'''^\s*\"([^"]+)\"\s+(\d+)\s+(\d+).*\n^\s*\"([^"]+)\"\s+(\d+)\s+(\d+).*$''', re.MULTILINE)
	re_h_tag = re.compile(r'''^\s*\">([^"]+)\"\s*^\s*\">([^"]+)\"''', re.MULTILINE)
	re_a_tag = re.compile(r'(\S)\s*(\d.*)')
	re_num = re.compile(r'\d+')
	
	# segments of the alignemnt of the form [s1, s2, e1, e2, id, score]
	segments = [];
	
	# for all tags found
	for tag, value in re_tag.findall(f.read()):
		
		if tag == 'd':
			# program parameters, substitution matrix etc
			pass
		elif tag == 's':
			# file names and positinons tag
			m = re_s_tag.search(value)
			file1 = m.group(1); start1 = m.group(2); end1 = m.group(3)
			file2 = m.group(4); start2 = m.group(5); end2 = m.group(6)
		elif tag == 'h':
			# fasta ids
			m = re_h_tag.search(value)
			fasta1 = m.group(1)
			fasta2 = m.group(2)
		elif tag == 'x':
			pass
		elif tag == 'm':
			pass
		elif tag == 'a':
			# alignment
			for a_tag, a_value in re_a_tag.findall(value):
				if a_tag == 's':
					# score for the alignment
					score_l = re_num.findall(a_value)
					assert len(score_l) == 1
					score = score_l[0]
				elif a_tag == 'l':
					# gap-free region
					align_l = re_num.findall(a_value)
					# add the score to the list
					align_l.append(score)
					# ->int
					align_l = map(int, align_l)
					# add to the list of segments
					segments.append(align_l);
				elif a_tag == 'b':
					pass
				elif a_tag == 'e':
					pass

	

	Alignment._transform_segments(segments, add=add, mul=mul)

	# create the alignment
	return Alignment(fasta1, file1, start1, end1, fasta2, file2, start2, end2, segments, overlap)

def writeblastz(align, filename, header='"a single line"'):
	f = open(filename, 'w')
	f.write('#:lav\n')
	f.write('d {\n')
	f.write('  "'+header+'"\n')
	f.write('}\n')
	f.write('#:lav\n')
	f.write('s {\n')
	f.write(' "'+align.file1+'" '+str(align.seq1_start)+' '+str(align.seq1_end)+'\n')
	f.write(' "'+align.file2+'" '+str(align.seq2_start)+' '+str(align.seq2_end)+'\n')
	f.write('}\n')
	f.write('h {\n')
	f.write(' ">'+align.seq1+'"\n')
	f.write(' ">'+align.seq2+'"\n')
	f.write('}\n')
	for seg in align.segments:
		f.write('a {\n')
		f.write(' s '+str(seg[alignment.SEG_SCORE])+'\n')
		f.write(' b '+str(seg[alignment.SEG_START1])+' '+str(seg[alignment.SEG_START2])+'\n')
		f.write(' e '+str(seg[alignment.SEG_END1])+' '+str(seg[alignment.SEG_END2])+'\n')
		f.write(' l '+str(seg[alignment.SEG_START1])+' '+str(seg[alignment.SEG_START2])+\
			' '+str(seg[alignment.SEG_END1])+' '+str(seg[alignment.SEG_END2])+\
			' '+str(seg[alignment.SEG_IDENT])+'\n')
		f.write('}\n')
	f.write('x {\n')
	f.write(' n 0\n')
	f.write('}\n')
	f.write('m {\n')
	f.write(' n 0\n')
	f.write('}\n')
	f.write('#:eof\n')
	f.close()

if __name__ == '__main__':
	for filename in sys.argv[1:]: print readblastz(filename)
