#!/usr/bin/python

import re
import seqtool
import sys

from readtool import skipto

True, False = 1==1, 1!=1

def readfasta(filename):
	f = open(filename)

	# create a dictionary
	d = dict()

	# whitespace regexp
	whitespace = re.compile(r'\s')
	
	seq_name = None
	
	# repeat for each sequence
	while True:
		
		# read sequence name/sequence
		m = skipto(f, r'^\s*>(?P<seq_name>.+)$|^(?P<seq_frag>\s*[^>].*)$')
		
		# end of the file
		if m == None: break
		
		# is this the name of new sequence?
		if m.group('seq_name') != None:
			seq_name = m.group('seq_name')
			
			# put an empty entry in the dictionary
			d[seq_name] = ''

		# is this a sequence fragment?
		if m.group('seq_frag') != None:
			
			seq_frag = m.group('seq_frag')

			# sanity check
			assert seq_name, "No sequence name provided in the fasta file"
			
			# remove whitespaces from the sequence
			seq_frag = whitespace.sub('', seq_frag);
			
			# join the fragment with the sequence
			d[seq_name] = d[seq_name] + seq_frag
	
	return d

if __name__ == '__main__':
	for filename in sys.argv[1:]: print readfasta(filename)
