#!/usr/bin/env python2.4

import sys
import getopt
import copy
import re

from readblastz import readblastz
from readblastz import writeblastz
from alignment import Alignment
from alignment import SEG_IDENT
from alignment import SEG_START1
from alignment import SEG_END1
from alignment import SEG_START2
from alignment import SEG_END2
from debug import Debug
        
if __name__ == '__main__':
    
    opts, args_left = getopt.getopt(sys.argv[1:],'e:a:cfiprv:xnb',['multiply=', 'add=', 'bed_multiply=', 'bed_add=', 'chr=', 'remove_overlap'])
    if len(opts) == 0 and len(args_left) == 0:
        #rint '         1         2         3         4         5         6         7         8'
        #rint '12345678901234567890123456789012345678901234567890123456789012345678901234567890'
        print 'Usage: ',sys.argv[0]
        print '-p ALIGN REGIONS'
        print '    prints no. residues covered by alignment'
        print '-f ALIGN REGIONS'
        print '    filters the alignment, restricting it to given region, output: _filt.blastz'
        print '    output file: *_filt.blastz'
        print '[-x] -v k ALIGN'
        print '    filters the alignments to those which have identity at least k.'
        print '    for identity exact k, use -x.'
        print '    output file: *_ident.blastz'
        print '-i ALIGN REGIONS'
        print '    reports maximal identity levels in the region'
        print '-c REGIONS'
        print '    number of residues in the region'
        print '[-r] reverse - check the second sequence of the alignment'
        print '-n ALIGN1 ALIGN2'
        print '    outputs parts of ALIGN1 inconsistent with ALIGN2'
        print '-b ALIGN --chr=CHROMOSOME_ID [--bed_multiply=v1] [--bed_add=v2]'
        print '    creates BED file based on the alignment, chr'
        print '    the color of annotation comes from identity level,'
        print '    after multiplying by v1 and adding v2'
#        print '--reportregions=REGIONS_FILE ALIGN'
#        print '    creates REGIONS file based on coverage of the alignment'
        print ''
        print 'ALIGN: -a alignment.blastz [--remove_overlap]'
        print '    --remove_overlap removes ambiguous matches which cover 1st sequence.'
        print '    Choses matches with higher identity level'
        print 'REGIONS: -e exons'
        print ''
        print '        OPTIONAL'
        print ''
        print '--multiply=value   multiply a value of identity level'
        print '--add=value        add a value to identity level'
        sys.exit(0)
    
    PARAM_ALIGN_VALUE = None
    PARAM_ALIGN_2ND_VALUE = None
    PARAM_EXONS_VALUE = None
    PARAM_FILTER = False
    PARAM_COUNT = False
    PARAM_PRINT = False
    PARAM_SECOND_SEQ = False
    PARAM_IDENT = False
    PARAM_OVERLAP = False
    PARAM_OVERLAP_EXACT = False
    PARAM_INCONSIST = False
    PARAM_BED = False
    PARAM_ADD = 0.0
    PARAM_MULTIPLY = 1.0
    PARAM_BED_ADD = 0.0
    PARAM_BED_MULTIPLY = 1.0
    PARAM_CHR = None
    PARAM_REPORT_REGIONS = False
    PARAM_REGIONS_OUT = None
    PARAM_REMOVE_OVERLAP = False
    for opt, param in opts:
        if opt == '-a':
            if PARAM_ALIGN_VALUE == None:
                PARAM_ALIGN_VALUE = param
            else:
                # second alignment
                PARAM_ALIGN_2ND_VALUE = param
        if opt == '-n':
            PARAM_INCONSIST = True
        if opt == '-e':
            PARAM_EXONS_VALUE = param
        if opt == '-c':
            PARAM_COUNT = True
        if opt == '-i':
            PARAM_IDENT = True
        if opt == '-x':
            PARAM_OVERLAP_EXACT = True
        if opt == '-v':
            PARAM_OVERLAP = True
            PARAM_OVERLAP_VALUE = int(param)
        if opt == '-f':
            PARAM_FILTER = True
        if opt == '-p':
            PARAM_PRINT = True
        if opt == '-r':
            PARAM_SECOND_SEQ = True
        if opt == '-b':
            PARAM_BED = True
        if opt == '--multiply':
            PARAM_MULTIPLY = float(param)
        if opt == '--add':
            PARAM_ADD = float(param)
        if opt == '--bed_multiply':
            PARAM_BED_MULTIPLY = float(param)
        if opt == '--bed_add':
            PARAM_BED_ADD = float(param)
        if opt == '--chr':
            PARAM_CHR = param
        if opt == '--remove_overlap':
            PARAM_REMOVE_OVERLAP = True
#        if opt == '--reportregions':
#            PARAM_REPORT_REGIONS = True
#            # there goes another 'if'!
#        if opt == '--reportregions':
#            PARAM_REGIONS_OUT = param
    
    if PARAM_BED: assert PARAM_CHR
    
    overlap = Alignment.ALLOW_OVERLAP
    
    program_params = ''.join([s+' ' for s in sys.argv])
    
    if PARAM_ALIGN_VALUE:
        orig_align = readblastz(PARAM_ALIGN_VALUE, overlap, add=PARAM_ADD, mul=PARAM_MULTIPLY)
        
        if PARAM_REMOVE_OVERLAP:
            # fragment segs
            orig_align.fragmentSegments(Debug.debug)
            # see Alignment.removeOverlap
            orig_align.sort(SEG_START1)
            # chose higher identity
            orig_align._remove_overlap(SEG_START1, SEG_END1, SEG_IDENT)
    
    if PARAM_ALIGN_2ND_VALUE:
        assert not PARAM_REMOVE_OVERLAP
        second_align = readblastz(PARAM_ALIGN_2ND_VALUE, overlap, add=PARAM_ADD, mul=PARAM_MULTIPLY)
    
    if PARAM_EXONS_VALUE:
#        print >>sys.stderr, "Reading segments"
        r = re.compile(r'^(\d+)\s+(\d+)\s*$')
        pos = []
        for line in open(PARAM_EXONS_VALUE):
            m = r.match(line)
            if m:
                pos.append([int(m.group(1)), int(m.group(2))])
                
        # merge regions
        pos = Alignment._regionsMerge(pos)
    
    if PARAM_FILTER or PARAM_OVERLAP or PARAM_INCONSIST:
        if PARAM_ALIGN_VALUE.endswith('.blastz'):
            PARAM_ALIGN_VALUE = PARAM_ALIGN_VALUE[:-len('.blastz')]
            
    if PARAM_FILTER:
        orig_align.segments = Alignment._regionsFilter(pos, orig_align.segments, SEG_START1, SEG_END1)
        writeblastz(orig_align, PARAM_ALIGN_VALUE+'_filt.blastz', program_params)
    elif PARAM_OVERLAP:
        nseg = []
        for seg in orig_align.segments:
            if seg[SEG_IDENT] == PARAM_OVERLAP_VALUE or (seg[SEG_IDENT] > PARAM_OVERLAP_VALUE and not PARAM_OVERLAP_EXACT):
                nseg.append(seg)
        orig_align.segments = nseg
        writeblastz(orig_align, PARAM_ALIGN_VALUE+'_ident.blastz', program_params)
    elif PARAM_COUNT:
        assert PARAM_EXONS_VALUE
        print Alignment.regionsLength(pos)
    elif PARAM_PRINT:
        assert PARAM_EXONS_VALUE
        assert PARAM_ALIGN_VALUE
        start = SEG_START1
        end = SEG_END1
        if PARAM_SECOND_SEQ:
            start = SEG_START2
            end = SEG_END2
        print Alignment._regionsCover(pos, orig_align.segments, start, end)
    elif PARAM_IDENT:
        assert PARAM_EXONS_VALUE
        assert PARAM_ALIGN_VALUE
        Alignment._regionsReportIdent(pos, orig_align.segments, SEG_START1, SEG_END1)
    elif PARAM_INCONSIST:
        assert PARAM_ALIGN_VALUE
        assert PARAM_ALIGN_2ND_VALUE
        orig_align.segments = Alignment._segmentsDiff(orig_align.segments, second_align.segments, SEG_START1, SEG_END1)
        writeblastz(orig_align, PARAM_ALIGN_VALUE+'_diff.blastz', program_params)
    elif PARAM_BED:
        # BED file coordinations are END-EXCLUSIVE (size=end-start)
        # for BLASTZ, coord are INCLUSIVE, size = end - start + 1
        assert PARAM_ALIGN_VALUE
        assert PARAM_CHR
        assert not PARAM_ALIGN_2ND_VALUE
        
        # prepare regions to merge
        regions = [ [ seg[SEG_START1],
                  seg[SEG_END1], 
                  seg[SEG_IDENT]] for seg in orig_align.segments ]
        
        # merge
        regions = Alignment._regionsStickyMerge(regions)
        
        # write
        f = open(PARAM_ALIGN_VALUE+'.bed', 'w')
        for r in regions:
            f.write('%s\t%d\t%d\t%s\t%d\n' 
                %
                (PARAM_CHR, 
                  r[0],
                  r[1]+1, 
                  r[2], 
                  r[2]*PARAM_BED_MULTIPLY + PARAM_BED_ADD) )
        f.close()
        
    elif PARAM_REPORT_REGIONS:
        assert PARAM_ALIGN_VALUE
        assert not PARAM_ALIGN_2ND_VALUE
        assert PARAM_REGIONS_OUT
        f = open(PARAM_REGIONS_OUT, 'w')
        for seg in orig_align.segments:
            f.write('%d\t%d\n' 
                %
                (seg[SEG_START1],
                  seg[SEG_END1] ) )
        f.close()
        