#!/usr/bin/python

import re

# GLOBAL - regexp for sequence name
reseqname = r'\S+' # r'\w(\w|\.|-)+'

# GLOBAL - a gap
gap = '.'

True, False = 1==1, 1!=1

def equalalignlen(d):
	# assert all the alignment lines have the same length 
	length = -1;
	for seq in d.itervalues():
		if length == -1: length = len(seq)
		assert len(seq) == length, 'Sequences not the same length: '+str(length)

def checkalign(align):
	'''Check the quality of alignment, and returns corrected version.
	
	For example columns only with gaps are removed.'''
		
	# assert all the alignment lines have the same length 
	equalalignlen(align)
	
	# get reference columns (might be inconsistent)
	refcol = _refcolumn(align, assert_=False)
	
	# correct the case of columns (may be still inconsistent),
	# according to the result from _refcolumn
	# ommiting gap-only columns (refcol[i] == None)
	for seqname in align.keys():
		seq = align[seqname]
		newseq = ''
		for i in xrange(0, len(seq)):
			if refcol[i] == True: newseq += seq[i].upper()
			if refcol[i] == False: newseq += seq[i].lower()
		align[seqname] = newseq
		
#	# create list of gap-only columns
#	gapcol = list()
#	while refcol.count(None) > 0:
#		pos = refcol.index(None)
#		gapcol.append(pos)
#		refcol[pos] = False
#		# remove gap-only column
#		del refcol[pos]
#		for seqname in align.keys():
#			# delete one character
#			seq = align[seqname]
#			seq = seq[:pos]+seq[pos+1:] 
#			align[seqname] = seq

	# the second call to the method: this time the method shouldn't fail
	refcol = _refcolumn(align, assert_=True) 
	
		

def refcolumn(align):
	'''Calculates which columns are reliable alignment 
	and asserts every uppercase column to be upppercase or dot in all sequences.
	
	Returns a list of booleans.'''
	
	refcol = _refcolumn(align)
	
	assert not None in refcol, 'There are columns in the alignment only with gaps '+ str(refcol)
	
	return refcol

def _refcolumn(align, assert_=True):
	'''Calculates which columns are reliable alignment 
	and asserts every uppercase column to be upppercase or dot in all sequences.
	
	Returns a list of booleans, None possible if column was gaps.
	Whenever there's inconsistency in case, sets value to false
	Does not modify the alignment.'''
	
	refcol = [None]*len(align.values()[0])
	
	for seqname in align.keys():
		seq = align[seqname]
		for i in xrange(0, len(seq)):
			if seq[i].isupper():
				if refcol[i] == None: refcol[i] = True
				if assert_: assert refcol[i], 'Inconsistent case in sequences at position '+str(i)
			if seq[i].islower():
#				if refcol[i] == None: refcol[i] = False
				if assert_: assert not refcol[i], 'Inconsistent case in sequences at position '+str(i)
				refcol[i] = False

	return refcol

def dotremove(seq):
	return re.compile(r'[.]').sub('', seq)
	

def checkmaligns(ref_align, align):
	'''Function asserts the multiple alignemnts
	
	Input is dictionary of multiple alignmetns'''
	
	ref_names = ref_align.keys()
	
	ref_names.sort
	names = align.keys()
	names.sort
	
	# check whether the protein names are the same
	assert ref_names == names, "Different proteins defined in reference and tested alignment"
	
	# check whether the proteins are the same
	for protein_name in ref_names:
		assert dotremove(ref_align[protein_name]).upper() == dotremove(align[protein_name]).upper(), "Different sequences of the same name"
	
	# construct the result matrix
	ref = [False]*len(align.values()[0])
	
	for seq in align.values():
		for i in xrange(0, len(seq)):
			if seq[i].isupper():
				ref[i] = True
				
	return ref
	
def seq2pos(gappedseq):
	'''Converts the gapped sequence into the sequence of positions.'''
	posseq = []
	k = 0
	for i in xrange(0, len(gappedseq)):
		if gappedseq[i] != '.':
			posseq.append(k)
			k += 1
		else:
			posseq.append('.')
			
	return posseq
	
def align2dictpos(align):
	'''Create the dictionary of alignments
	with positions in the sequence instead of letters.'''

	result = dict()
	for name in align.iterkeys():
		result[name] = seq2pos(align[name])
		
	return result

def seq2slots(gappedseq):
	'''Converts the gapped sequence into the sequence of positions.'''
	slots = []
	for i in xrange(0, len(gappedseq)):
		if gappedseq[i] != '.':
			slots.append(i)
			
	return slots
	
def align2dictslots(align):
	'''Create the dictionary of sequences based on
	positions of letters in the alignment.'''
	
	result = dict()
	for name in align.iterkeys():
		result[name] = seq2slots(align[name])
		
	return result
	
def iscorealignpair(letter1, letter2):
	'''Determines whether the letters are part of the core alignment.'''
	return letter1.isupper() and letter2.isupper()
	
def iscorealign(align, seq1, seq2, pos):
	'''Determines whether the letters are part of the core alignment.'''
	return iscorealignpair(align[seq1][pos], align[seq2][pos])
	