#!/usr/bin/env python2.4

# $Author: radek $

import sys
import getopt
import copy

from readblastz import readblastz
from readblastz import writeblastz
from alignment import Alignment
from alignment import SEG_IDENT
from debug import Debug
        
from alignment import SEG_START1

# use a hack: set identity to absurdly high value
# to detect overlap with original alignment
ABSURD_VALUE = 1000

if __name__ == '__main__':
    
    opts, args_left = getopt.getopt(sys.argv[1:],'amrv:sncf:di',[])
    if len(opts) == 0 and len(args_left) == 0:
        #rint '         1         2         3         4         5         6         7         8'
        #rint '12345678901234567890123456789012345678901234567890123456789012345678901234567890'
        print 'Aubergene $Revision: 1.16 $ $Date: 2006/02/07 13:37:50 $'
        print 'Usage: '
        print sys.argv[0], '[ -r | -v k ] [-s | -n] [ -a | -m [-f k] [-d] [-i] ] seq1 seq2 inter1 inter2 ... suffix'
        print ''
        print '    where'
        print '        seq1, seq2 - a pair of sequences to align'
        print '        inter1, inter2, ... - intermediate sequences to use'
        print ''
        print '    Parameters:'
        print '    -a realigns the sequences using the information from intermediate'
        print '       alignments (i.e. i.e., seq1-interX-seq2)'
        print '    -m outputs the alignment the original together with all the intermediate'
        print '       seq1-interX-seq2 alignments'
        print '        -d do not read the direct alignment'
        print '        -i do not calculate trivial transitive alignments'
        print '            (i.e. human-human-mouse and human-mouse-mouse) '
        print '    -v k outputs the alignments which were only validated by at least k'
        print '       intermediate alignments'
        print '    -r recovers the original alignment using intermediate ones. '
        print '       Has to be used with -m or -a options'
        print '    -s rescale identity levels to be between 50-100 (for better visibility '
        print '       in genome browsers). Default for -m and -v'
        print '    -n for each segment indicate by how many transitive alignments it was'
        print '       supported. Indicate this value as "identity level" in the final'
        print '       alignment. Without this option identity levels are rescalled to 100%.'
        print '    -c report overlapping matches.'
        print ''
        print '    See readme.txt for more.'
        sys.exit(0)
    
    PARAM_ALIGN = False
    PARAM_MERGE = False
    PARAM_VALIDATE = False
    PARAM_RECOVER = False
    PARAM_NON_TRIVIAL = False
    PARAM_SKIP_DIRECT = False
    PARAM_IDENT_RESCALE = True
    PARAM_NO_IDENT_RESCALE = False
    PARAM_REPORT_OVERLAP = False
    for opt, param in opts:
        if opt == '-a':
            PARAM_ALIGN = True
        if opt == '-m':
            PARAM_MERGE = True
        if opt == '-d':
            PARAM_SKIP_DIRECT = True
        if opt == '-i':
            PARAM_NON_TRIVIAL = True
        if opt == '-r':
            assert not PARAM_VALIDATE
            PARAM_RECOVER = True
        if opt == '-s':
            PARAM_NO_IDENT_RESCALE = False
            PARAM_IDENT_RESCALE = True
        if opt == '-n':
            PARAM_IDENT_RESCALE = False
            PARAM_NO_IDENT_RESCALE = True
        if opt == '-c':
            PARAM_REPORT_OVERLAP = True
        if opt == '-v':
            assert not PARAM_RECOVER
            PARAM_VALIDATE = True
            PARAM_VALIDATE_VALUE = int(param)
    
    # params sanity check
    assert (not PARAM_RECOVER) or (PARAM_ALIGN or PARAM_MERGE)
    
    overlap = Alignment.ALLOW_OVERLAP
    
    program_params = ''.join([s+' ' for s in sys.argv])
    
    seq1_file = args_left[0]
    seq2_file = args_left[1]
    suffix = args_left[-1]
    trans_files = args_left[2:len(args_left)-1]
    print 'Aligning',seq1_file,'with',seq2_file,'using',trans_files


    # read the original alignment
    print >>sys.stderr, "Reading main alignment..."
    orig_align = readblastz(seq1_file+'_'+seq2_file+suffix, overlap)
    if not PARAM_SKIP_DIRECT:
        print >>sys.stderr, "Writing main alignment..."
        writeblastz(orig_align, seq1_file+'_'+seq2_file+'_'+'orig'+suffix, program_params)
    else:
        print >>sys.stderr, "Skipping the direct alignment"
        orig_align.segments = []
    
    # hide the original alignment
    if (PARAM_RECOVER):
        orig_align_recover = copy.copy(orig_align)
        orig_align.segments = []
    
#    # sanity
#    if Debug.debug:
#        _debug_orig_coverage = [ orig_align.scoreCoverageCountSeq1(), orig_align.scoreCoverageCountSeq2() ]

    # calculate the whole list of new alignments
    print >>sys.stderr, "Reading alignments..."
    trans_aligns = []
    
    # calculate the new alignemnt using intermediate sequence
    for inter_file in trans_files: 
        if PARAM_NON_TRIVIAL and (inter_file == seq1_file or inter_file == seq2_file):
            print >>sys.stderr, "Skipping trivial alignments"
        else:
            seq1_inter = readblastz(seq1_file+'_'+inter_file+suffix, overlap)
            inter_seq2 = readblastz(inter_file+'_'+seq2_file+suffix, overlap)
            seq1_inter_seq2 = Alignment.transitive(seq1_inter, inter_seq2, overlap)
            writeblastz(seq1_inter_seq2, seq1_file+'_'+inter_file+'_'+seq2_file+suffix, program_params)
            trans_aligns.append(seq1_inter_seq2)

    # for the purpose of verification, set identity to 1 so it becomes additive,
    # and will represent the number of overlapping sequences
    if PARAM_RECOVER or PARAM_MERGE:
        
        Alignment.segmentsSetValue(orig_align.segments, SEG_IDENT, 1)
        for align in trans_aligns:
            Alignment.segmentsSetValue(align.segments, SEG_IDENT, 1)
            
    if PARAM_VALIDATE: # warning: before might have been set already because of merge!
        
        Alignment.segmentsSetValue(orig_align.segments, SEG_IDENT, ABSURD_VALUE)
        for align in trans_aligns:
            Alignment.segmentsSetValue(align.segments, SEG_IDENT, 1)
            
    # merge/realign
    if (PARAM_ALIGN or PARAM_MERGE):
        print >>sys.stderr, "Merging..."
        # merge alignments
        i = 0
        for align in trans_aligns:
            i += 1
            print >>sys.stderr, "Merging "+str(i)+" of "+str(len(trans_aligns))
            orig_align.merge(align)
            
    # Realigning
    if (PARAM_ALIGN):
        print >>sys.stderr, "Realigning..."
        orig_align.realign()
        
    # calculate which part of the original alignment has been recovered
    if PARAM_RECOVER:
        
        assert orig_align_recover
        
        # use a hack: set identity to absurdly high value
        ABSURD_VALUE = 1000
        Alignment.segmentsSetValue(orig_align_recover.segments, SEG_IDENT, ABSURD_VALUE)
        
        # count the transitive residues
        r_trans = orig_align.countResidues(0)
        
        # merge the alignment
        orig_align.merge(orig_align_recover)
        
        # how many nucleotides were confirmed?
        
        # count all the residues
        r_recov = orig_align_recover.countResidues(0)
        
        # count residues confirmed by alignments
        r_conf = orig_align.countResidues(ABSURD_VALUE+1)
        
        print str((r_conf * 100) / r_recov) + '% of original alignment confirmed'
        print str((r_conf * 100) / r_trans) + '% of transitive alignment useful'
        
    elif PARAM_VALIDATE:
        
        print >>sys.stderr, "Validating..."
        
        # validate the original alignment with the transitive ones
        #orig_align.validate(trans_aligns)
        
        # filter the alignment to the original one
        nseg = []
        for seg in orig_align.segments:
            if seg[SEG_IDENT] >= ABSURD_VALUE:
                seg[SEG_IDENT] -= ABSURD_VALUE
                nseg.append(seg)
        orig_align.segments = nseg
        
    if PARAM_REPORT_OVERLAP or PARAM_VALIDATE:
        print >>sys.stderr, 'residues confirmed at level'
        inc = 0
        if not PARAM_VALIDATE: inc = 1
        for x in xrange(0+inc, len(trans_files)+1+inc):
            r_conf = orig_align.countResidues(x)
            print x, r_conf

    if (PARAM_IDENT_RESCALE or PARAM_VALIDATE or PARAM_MERGE) and not PARAM_NO_IDENT_RESCALE:
        print >>sys.stderr, "Rescaling..."
        # change the identity field to be between 50 and 100
        orig_align.rescaleident(50, 100)
        
#        # sanity
#        if Debug.debug:
#            _debug_coverage = [ orig_align.scoreCoverageCountSeq1(), orig_align.scoreCoverageCountSeq2() ]
#            Alignment.assertCoverages(_debug_coverage, _debug_orig_coverage)
#            _debug_coverage = _debug_orig_coverage = None
    
    
    print >>sys.stderr, 'Sanity checking of the alignment...'
    Alignment._resolveOverlap(orig_align.segments, SEG_START1)
    
    print >>sys.stderr, "Writing..."
    # write the new alignment
    writeblastz(orig_align, seq1_file+'_'+seq2_file+'_'+'valid'+suffix, program_params)
